import os
from google.cloud import bigquery

dataset_name = os.getenv("dataset_name")
table_name = os.getenv("table_name")


def fn_write_file_gcs_bgq(event, context):
    """Background Cloud Function to be triggered by Cloud Storage.
    Args:
        event (dict):  The dictionary with data specific to this type of event.
        context (google.cloud.functions.Context): Metadata of triggering event.
    """
    try:
        # Bigquery instance
        bigquery_client = bigquery.Client()

        if event["name"].split("/")[0] == "public_dataset":
            uri = f"gs://{event['bucket']}/{event['name']}"

            table_ref = bigquery_client.get_dataset(dataset_ref=dataset_name).table(table_id=table_name)
            table = bigquery_client.get_table(table=table_ref)

            job_config = bigquery.job.LoadJobConfig(
                schema=table.schema,
                source_format=bigquery.SourceFormat.CSV,
                field_delimiter=","
            )
            load_job = bigquery_client.load_table_from_uri(source_uris=uri, destination=table, job_config=job_config)
            load_job.result()

    except Exception as err:
        print(err)



import boto3
import zipfile
import os
from io import BytesIO


s3 = boto3.client("s3")


def handler(event, context):

    bucket = event["bucketName"]
    input_key = event["inputKey"]
    output_prefix = event["outputPrefix"]

    s3_resource = boto3.resource("s3")

    _, ext = os.path.splitext(input_key)

    if ext == ".zip":

        zip_obj = s3_resource.Object(bucket_name=bucket, key=input_key)
        print(zip_obj)
        buffer = BytesIO(zip_obj.get()["Body"].read())

        z = zipfile.ZipFile(buffer)
        for filename in z.namelist():
            file_info = z.getinfo(filename)
            s3_resource.meta.client.upload_fileobj(
                z.open(filename),
                Bucket=bucket,
                Key=output_prefix + os.path.basename(filename),
            )

    return


import os
from concurrent.futures import TimeoutError
from google.cloud import pubsub_v1

path = os.path.dirname(os.path.dirname(__file__))
os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = f"{path}/config/service-account.json"

project_id = ""
subscription_id = ""
subscriber = pubsub_v1.SubscriberClient()


def get_message(message):
    print(f"{message}")
    message.ack()


subscription_path = subscriber.subscription_path(project_id, subscription_id)
streaming_pull = subscriber.subscribe(subscription_path, callback=get_message)

with subscriber:
    try:
        streaming_pull.result()
    except TimeoutError:
        streaming_pull.cancel()  # Trigger the shutdown.
        streaming_pull.result()  # Block until the shutdown is complete.



import click
import os
from .validation import get_operational_system, check_version_python, check_python_compile
from .module_writer import modules_autocomplete, write_module, create_module_from_diff
from .cli_aux import get_modules_to_add, get_cli_decorators, get_autocomplete
from .__init__ import __version__ as version

from datetime import datetime

LOCAL_PATH = os.getcwd()

logo = r"""

 _ __ ___  _ __  _   _
| '__/ _ \| '_ \| | | |
| | | (_) | | | | |_| |
|_|  \___/|_| |_|\__, |
                 |___/
v{}
""".format(
    version
)


@click.group()
def cli():
    pass


@cli.command()
def info():
    """
    Checks that Rony is correctly installed
    """
    click.echo(logo)


@cli.command()
@click.argument("project_name")
@click.option(
    "--provider", "-p", default="aws", autocompletion=get_autocomplete("new", "provider")
)
@click.option("-y", "--autoconfirm", is_flag=True)
@click.option("-e", "--excludeci", is_flag=True)
@click.pass_context
def new(ctx, project_name, **kwargs):
    """Create a new Rony project

    Args:
        project_name (str): Name to project
        kwargs (dict): Flags and options
    """

    # Getting all modules to be added
    module_names = get_modules_to_add("new", kwargs, ctx)

    if not module_names:
        raise click.UsageError("Invalid parameters. Please review your command")

    click.echo(f"Creating project {project_name}")

    # Creating project directory
    os.makedirs(os.path.join(LOCAL_PATH, project_name))

    # Inputs to be passed to all modules
    custom_inputs = {
        "project_name": project_name,
        "project_start_date": datetime.today().strftime("%B %d, %Y"),
    }

    # Running modules
    for module_name in sorted(set(module_names), key=module_names.index):
        write_module(
            os.path.join(LOCAL_PATH, project_name),
            module_name,
            True,
            custom_inputs,
        )

    os.chdir(project_name)
    env_name = f"{project_name}_env"
    os.environ["ENV_NAME"] = env_name
    check_python_compile()

    # Create git repo
    os.system("git init")

    # Give execution permissions to CI/scripts folder
    recursive_chmod("./CI/scripts", 0o755)

    print(
        "A git repository was created. You should add your files and make your first commit.\n"
    )


for dec in get_cli_decorators("new"):
    new = dec(new)


@click.argument("image_name")
@cli.command()
def build(image_name):
    """Build a docker image with given image_name. Only run if you have docker installed.
    One should be at the root directory

    Args:
        image_name (str): Name to image docker
    """
    if not os.path.exists("Dockerfile"):
        click.echo("You gotta have a Dockerfile file.")
    else:
        os.system(f"docker build -t {image_name} .")


@click.argument("image_name")
@cli.command()
def run(image_name):
    """Run a container with given image_name.
    Only run if you have docker installed.]

    Args:
        image_name (str): Name to image docker
    """
    if not os.path.exists("Dockerfile"):
        click.echo("You gotta have a Dockerfile file")
    else:
        os.system(f"docker run --rm {image_name}")


@click.argument("module_name", type=click.STRING, autocompletion=modules_autocomplete)
@click.option("-y", "--autoconfirm", is_flag=True)
@cli.command()
def add_module(module_name, autoconfirm):
    """Add new module to rony project
    One should be at the root directory

    Args:
        module_name (str): Name of the module to be added
    """
    write_module(LOCAL_PATH, module_name, autoconfirm)

    recursive_chmod("./CI/scripts", 0o755)


@click.argument("module_name", type=click.STRING, autocompletion=modules_autocomplete)
@cli.command()
def diff_2_module(module_name):
    """Add new module to rony project
    One should be at the root directory

    Args:
        module_name (str): Name of the module to be added
    """
    create_module_from_diff(module_name)


def recursive_chmod(path, mode):
    for dirpath, dirnames, filenames in os.walk(path):
        os.chmod(dirpath, mode)
        for filename in filenames:
            os.chmod(os.path.join(dirpath, filename), mode)


@click.argument(
    "provider_name", type=click.STRING, autocompletion=get_autocomplete("new", "provider")
)
@click.option("-y", "--autoconfirm", is_flag=True)
@cli.command()
@click.pass_context
def add_provider(ctx, provider_name, **kwargs):
    """Add new provider to rony project
    One should be at the root directory

    Args:
        provider_name (str): Name of the provider to be added
    """

    kwargs["provider"] = provider_name
    kwargs["excludeci"] = True
    module_names = get_modules_to_add("new", kwargs, ctx)

    # Inputs to be passed to all modules
    custom_inputs = {
        "provider_name": provider_name,
        "provider_start_date": datetime.today().strftime("%B %d, %Y"),
    }

    for module_name in sorted(set(module_names), key=module_names.index):
        write_module(
            os.path.join(LOCAL_PATH),
            module_name,
            custom_inputs,
        )

    recursive_chmod("./CI/scripts", 0o755)



import click
import os
from packaging import version
from datetime import datetime
import json
from jinja2 import Template
import rony

import importlib
import pkgutil
import subprocess
from unidiff import PatchSet

plugins_dirs = [
    importlib.import_module(name).__path__[0]
    for finder, name, ispkg in pkgutil.iter_modules()
    if name.startswith("rony_")
]

EXCLUDE_DIRS = ["__pycache__"]


def get_modules():
    def get_module_info(module_name, parent_dir):
        config_file = os.path.join(parent_dir, f"{module_name}.json")
        if os.path.exists(config_file):
            with open(config_file, "r") as f:
                data = json.load(f)
            return data
        else:
            return {}

    def comp_modules_version(info1, info2):
        v1 = version.parse(info1.get("version", "0.0.0"))
        v2 = version.parse(info2.get("version", "0.0.0"))
        return v1 < v2

    parent_dirs = [rony.__path__[0]] + plugins_dirs
    modules_dirs = [os.path.join(parent, "module_templates") for parent in parent_dirs]
    if os.path.exists(os.path.join(os.path.expanduser("~"), "MyRonyModules")):
        modules_dirs += [os.path.join(os.path.expanduser("~"), "MyRonyModules")]

    module_paths = {}
    module_info = {}
    module_desc = {}

    for modules_dir in reversed(modules_dirs):
        for module_name in next(os.walk(modules_dir))[1]:

            tmp_info = get_module_info(module_name, modules_dir)
            if (module_name in module_paths.keys()) and comp_modules_version(
                tmp_info, module_info[module_name]
            ):
                continue
            else:
                module_paths[module_name] = os.path.join(modules_dir, module_name)
                module_info[module_name] = tmp_info
                module_desc[module_name] = tmp_info.get("info", "")

    return module_paths, module_desc


def modules_autocomplete(ctx, args, incomplete):
    """Get list of modules available for installation

    Args:
        ctx:
        args:
        incomplete:
    """

    _, module_desc = get_modules()
    return [
        (key, module_desc[key])
        for key in module_desc.keys()
        if (incomplete in key) and key[:2] != "__"
    ]


def write_module(
    LOCAL_PATH,
    module_name,
    autoconfirm=False,
    custom_inputs={},
):
    """Copy files to project

    Args:
        LOCAL_PATH (str): Local Path
        project_name (str): Project Name
    """

    def get_inputs(input_info, autoconfirm=False):
        input_data = {}
        for info in input_info:
            if autoconfirm:
                input_data[info[0]] = info[1]
            else:
                input_data[info[0]] = click.prompt(info[2], default=info[1])
        return input_data

    module_paths, _ = get_modules()
    module_path = module_paths[module_name]
    config_file = os.path.join(os.path.dirname(module_path), f"{module_name}.json")

    # Load config file
    if os.path.exists(config_file):
        with open(config_file, "r") as f:
            data = json.load(f)
    else:
        data = {"input_info": []}

    # Request input data
    data["inputs"] = get_inputs(data["input_info"], autoconfirm)
    data["inputs"].update(custom_inputs)

    # Process files
    files_to_create = {}
    files_to_append = {}
    dirs_to_create = []

    for dir_path, dirs, files in os.walk(module_path):

        dir_name = os.path.basename(dir_path)
        rel_path = os.path.relpath(dir_path, module_path)
        local_dir_path = os.path.join(LOCAL_PATH, rel_path)

        if dir_name in EXCLUDE_DIRS:
            continue

        for d in dirs:
            if d in EXCLUDE_DIRS:
                continue
            local_d_path = os.path.join(local_dir_path, d)
            if not os.path.exists(local_d_path):
                dirs_to_create.append(local_d_path)

        for f in files:

            if f == ".ronyignore":
                continue

            outputText = open(os.path.join(dir_path, f), "r", encoding="latin-1").read()

            if ".tpl." in f:
                template = Template(outputText)
                outputText = template.render(**data)

            local_f_path = os.path.join(local_dir_path, f.replace(".tpl.", "."))
            if os.path.exists(local_f_path):
                files_to_append[local_f_path] = outputText
            else:
                files_to_create[local_f_path] = outputText

    # Show changes to user and ask for permission

    if not autoconfirm:
        click.secho("DIRECTORIES TO BE CREATED:", fg="green", bold=True)
        for key in dirs_to_create:
            click.secho(key, fg="green")

        click.secho("FILES TO BE CREATED:", fg="green", bold=True)
        for key, text in files_to_create.items():
            click.secho(key, fg="green")
            click.secho("\t+  ".join(("\n" + text).splitlines(True)) + "\n")

        click.secho("FILES TO BE APPENDED:", fg="yellow", bold=True)
        for key, text in files_to_append.items():
            click.secho(key, fg="yellow")
            click.secho("\t+  ".join(("\n" + text).splitlines(True)) + "\n")

    if (not autoconfirm) and (
        not click.confirm("Do you want to continue?", default=True, abort=True)
    ):
        return

    # Print instructions
    if not autoconfirm:
        click.secho("INSTRUCTIONS", fg="green", bold=True)
        click.secho("\n".join(data.get("instructions", [])))
        click.secho("DEVELOPED BY:", fg="green", bold=True)
        click.secho(", ".join(data.get("developers", [])))

    # Create and append files and dirs

    for directory in dirs_to_create:
        if not os.path.exists(directory):
            os.makedirs(directory)

    for key, text in files_to_create.items():
        directory = os.path.dirname(key)
        if not os.path.exists(directory):
            os.makedirs(directory)
        with open(key, "wb") as f:
            f.write(text.encode("latin-1"))

    for key, text in files_to_append.items():
        with open(key, "ab") as f:
            f.write(("\n\n" + text).encode("latin-1"))


def create_module_from_diff(module_name):

    if (
        subprocess.call(
            ["git", "branch"], stderr=subprocess.STDOUT, stdout=open(os.devnull, "w")
        )
        != 0
    ):
        click.Abort("Current directory is not a git repository")

    patch_file = f".rony_{module_name}.patch"
    module_path = os.path.join(os.path.expanduser("~"), "MyRonyModules", module_name)

    p = subprocess.call(["git", "diff", "--no-prefix"], stdout=open(patch_file, "w"))

    patch_set = PatchSet.from_filename(patch_file)

    for patched_file in patch_set:

        file_path = patched_file.path
        added_lines = []
        for hunk in patched_file:
            for line in hunk:
                if line.is_added:
                    added_lines.append(line.value)

        module_file_path = os.path.join(module_path, file_path)
        module_dir_path = os.path.dirname(module_file_path)

        if not os.path.exists(module_dir_path):
            os.makedirs(module_dir_path)

        with open(module_file_path, "w") as f:
            f.write("".join(added_lines))

    os.remove(patch_file)

    info = click.prompt("Please your modules description", default="")
    inst = click.prompt(
        "Please instructions to be displayed to the users after they add this module",
        default="",
    )
    developer = click.prompt("Please enter your email", default="")

    with open(module_path + ".json", "w") as f:
        f.write(
            json.dumps(
                {
                    "info": info,
                    "instructions": [inst],
                    "developers": [developer],
                    "input_info": [],
                    "version": "0.0.0",
                },
                sort_keys=True,
                indent=4,
                separators=(",", ": "),
            )
        )



import importlib
import pkgutil
import click

plugins = [
    importlib.import_module(name)
    for finder, name, ispkg in pkgutil.iter_modules()
    if name.startswith("rony_")
]


def get_cli_decorators(command):
    """Return the decorators to be added to commands

    Args:
        command (str): Name of the command ran
    """

    all_decorators = []

    for plugin in plugins:
        if hasattr(plugin, "cli_aux"):
            all_decorators += plugin.cli_aux.get_cli_decorators(command)

    return all_decorators


def get_modules_to_add(command, opts, ctx):
    """Return the modules to be added base on the command ran

    Args:
        command (str): Name of the command ran
        opts (dict): Dict with options and flags passade to the command line
    """

    all_modules = []

    if command == "new":
        if opts["excludeci"] == False:
            all_modules += ["CI_workflows"] 

        if opts["provider"] == "aws":

            all_modules += [ "__AWS_BASE__"]

            if opts["autoconfirm"]:
                all_modules += [
                    "aws_simple_storage_service",
                    "aws_glue_crawler",
                    "aws_lambda_function",
                ]
            else:
                if click.confirm("Add S3 module?", default=True):
                    all_modules.append("aws_simple_storage_service")
                if click.confirm("Add GLUE CRAWLER module?", default=True):
                    all_modules.append("aws_glue_crawler")
                if click.confirm("Add LAMBDA FUNCTION module?", default=True):
                    all_modules.append("aws_lambda_function")

        if opts["provider"] == "gcp":
            
            all_modules += ["__GCP_BASE__"]

            if opts["autoconfirm"]:
                all_modules += [
                    "gcp_cloud_storage",
                    "gcp_bigquery",
                    "gcp_cloud_function",
                    "gcp_pubsub",
                ]
            else:
                if click.confirm("Add CLOUD_STORAGE module?", default=True):
                    all_modules.append("gcp_cloud_storage")
                if click.confirm("Add BIGQUERY module?", default=True):
                    all_modules.append("gcp_bigquery")
                if click.confirm("Add CLOUD FUNCTION module?", default=True):
                    all_modules.append("gcp_cloud_function")
                if click.confirm("Add PUBSUB module?", default=True):
                    all_modules.append("gcp_pubsub")

    for plugin in plugins:
        if hasattr(plugin, "cli_aux") and hasattr(plugin.cli_aux, "get_modules_to_add"):
            all_modules += plugin.cli_aux.get_modules_to_add(command, opts, ctx)

    return all_modules


def get_autocomplete(command, opt_name):
    def autocomplete(ctx, args, incomplete):
        return [opt for opt in options if (incomplete in opt[0])]

    options = []

    if command == "new":
        if opt_name == "provider":
            options.append(("aws", "AWS provider"))
            options.append(("gcp", "GCP provider"))

    for plugin in plugins:
        if hasattr(plugin, "cli_aux") and hasattr(plugin.cli_aux, "get_autocomplete"):
            options += plugin.cli_aux.get_autocomplete(command, opt_name)

    return autocomplete



import os
import json
from google.cloud import pubsub_v1
from fake_web_events import Simulation

path = os.path.dirname(os.path.dirname(__file__))
os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = f"{path}/config/service-account.json"

topic = ""
publisher = pubsub_v1.PublisherClient()
simulation = Simulation(user_pool_size=100, sessions_per_day=1000)
events = simulation.run(duration_seconds=60)


def put_message(data):
    data = json.dumps(data) + "\n"
    response = publisher.publish(topic=topic, data=data.encode("utf-8"))
    print(data)
    return response.result()


for event in events:
    put_message(event)
